import { csv, select } from 'd3';
import { scatterPlot } from './scatterPlot';

// Defining the dataset
const csvUrl = [
  'https://gist.githubusercontent.com/chakravarthulaharsha/310d954b3fdfb2741ebedc8f3acd2804/raw/254d8a3797278cf95a6758a1b5a415d79adc019d/penguin.csv', // File name
].join('');

// loading the data into each parameter
const parseRow = (d) => {
  d.bill_length = +d.bill_length;
  d.bill_width = +d.bill_depth;
  d.flipper_length = +d.flipper_length;
  d.body_mass = +d.body_mass;
  return d;
};

// Define the size of the image
const width = window.innerWidth;
const height = window.innerHeight;
const svg = select('body')
  .append('svg')
  .attr('width', width)
  .attr('height', height);

// defining all the constants
const xValue = (d) => d.flipper_length;
const yValue = (d) => d.body_mass;
const rxValue = (d) => d.bill_length;
const ryValue = (d) => d.bill_depth;
const speciesValue = (d) => d.species;
const colorValue = d => d.species;
const colorLabel = 'Species';

// Wait for data to be loaded 
const main = async () => {
  const data = await csv(csvUrl, parseRow);
  const x = xValue((d) => d.body_mass);
  const y = yValue((d) => d.bill_length);
  
  // determine the color of the mark based on the specie
  const marker = (d) => {
        	switch(d){
            case 'Adelie': //label names on top
              return 'skyblue';  // colours for each type
            case 'Gentoo':
              return 'lightgreen';
          	case 'Chinstrap':
              return 'red';
            default:
              return 'orange';
          }
        }
  
  //Defining the mark
  const marks = data.map((d) => ({
          x: xValue((d) => d.body_mass),
          y: yValue((d) => d.bill_length),
          species: speciesValue(d),
          color: marker(speciesValue(d)),
        	rx: rxValue(d),
          ry: ryValue(d),
                      
        }));
  
  
//structure of scatterplot  is defined
  const plot = scatterPlot()
    .width(width)
    .height(height)
    .data(await csv(csvUrl, parseRow))
    .xValue((d) => d.body_mass)
    .yValue((d) => d.bill_length)
    .margin({
      top: 28,
      right: 53,
      bottom: 82,
      left: 88,

    })
    .radius(5);
  
  
 const g = svg.append('g')
          .attr('transform', `translate(${100},${100})`);

      const colorLegendG = g.append('g')
          .attr('transform', `translate(${100}, 150)`);





//shape of that scattered data 
svg
          .selectAll('circle')
          .data(marks)
          .join('circle')
          .attr('cx', (d) => d.x)
          .attr('cy', (d) => d.y)
        	.attr('fill', (d) => d.color)
        	.attr('rx', (d) => (d.rx))
        	.attr('ry', (d) => (d.ry));

     
    
  
  const columns = [
    'body_mass',
    'bill_depth',
    'flipper_length',
    'bill_length',
  ];

  
  let i = 0;
  setInterval(() => {
    plot.xValue((d) => d[columns[i % columns.length]]);  
		svg.selectAll('text').remove();
    svg.call(plot);
    
      colorLegendG.append('text')
          .attr('class', 'legend-label')
          .attr('x', -70)
          .attr('y', -210)
          .style("font-style","italic")
          .style("stroke","violet")
    			.style("font-weight","bold")
    			.style("font-style","italic")
          .text(colorLabel);

   
// display Title 
svg.append("text").attr("x", 250).attr("y", 20).text("Reusable scatter plot for Palmer penguins").style("font-size", "40px").attr("alignment-baseline","middle").attr('fill','lightgreen').style("font-style","italic")
        	.style("text-decoration","underline")

   
// Adding  labels in the legend and its style 
svg.append("text").attr("x", 150).attr("y", 70).text("Setosa").style("font-size", "20px").attr("alignment-baseline","middle")
svg.append("text").attr("x", 150).attr("y", 100).text("Gentoo").style("font-size", "20px").attr("alignment-baseline","middle")
svg.append("text").attr("x", 150).attr("y", 130).text("Chinstrap").style("font-size", "20px").attr("alignment-baseline","middle")
svg.append("text").attr("x", 150).attr("y", 160).text("other").style("font-size", "20px").attr("alignment-baseline","middle")
  
    svg
        	.append("text")
        	.attr('transform', 'rotate(-90)')
        	.attr('x',0 - (height /2))
        	.attr("y",  height/12)
    			.attr('fill', 'red')
        	.style("text-anchor", "middle")
        	.style("font-style","italic")
        	.style("font-weight","bold")
        	.text("Bill Length");
    
    // Add points for legend
    svg.append("circle").attr("cx",130).attr("cy",70).attr("r", 6).style("fill", "lightblue")
		svg.append("circle").attr("cx",130).attr("cy",100).attr("r", 6).style("fill", "lightgreen")
    svg.append("circle").attr("cx",130).attr("cy",130).attr("r", 6).style("fill", "red")
		svg.append("circle").attr("cx",130).attr("cy",160).attr("r", 6).style("fill", "orange")
   // plot Body Mass on x-axis
    if(i % columns.length==0) 
    svg
        	.append("text")
        	.attr("x", width / 2 )
      		.attr("y",  480 )
    			.attr('fill','red')
        	.style("text-anchor", "middle")
        	.style("font-style","italic")
        	.style("font-weight","bold")
      		.text("Body Mass");
     // plot Bill Depth on x-axis
    else if(i % columns.length==1) 
    svg
        	.append("text")
        	.attr("x", width / 2 )
      		.attr("y",  480 )
    			.attr('fill','red')
        	.style("text-anchor", "middle")
        	.style("font-style","italic")
        	.style("font-weight","bold")
      		.text("Bill Depth");
     // plot Flipper Length on x-axis
    else if(i % columns.length==2) 
    svg
        	.append("text")
        	.attr("x", width / 2 )
      		.attr("y",  520 )
    			.attr('fill','red')
        	.style("text-anchor", "middle")
        	.style("font-weight","bold")
      		.text("Flipper Length");
     // plot Bill Length on x-axis
    else 
    svg
        	.append("text")
        	.attr("x", width / 2 )
      		.attr("y",  540 )
        	.style("text-anchor", "middle")
        	.style("font-style","italic")
        	.style("font-weight","bold")
      		.text("Bill Length");
    i++;
    // every 5 seconds changes the visualization
  }, 5000);
};
main();




